#include "adder.h"
//#include "monitor2.h"


// The constructor
adder::adder(sc_module_name module_name, int num_mons):sc_module(module_name) {

  SC_THREAD( driver );
  sensitive << input1 << input2;
  
  _a = 0;
}


void
adder::driver() {
  DP->dprint(4, "driver(): Start execution\n");

  while(true) {
    DP->dprint(5, "driver(): Suspending until the values on the inputs change\n");
    wait(input1.value_changed_event() | input2.value_changed_event());

    DP->dprint(5, "driver(): The values on the inputs have changed\n");
    DP->dprint(5, "driver(): Reading from the input ports\n");
    int addent1 = input1.read();
    int addent2 = input2.read();

#ifdef MONITOR_2
    for (int i = 0; i < num_monitors; i++) {
      DP->dprint(5, "driver(): Received inputs. Notifying monitor i=%d\n", i);
      monitor2* prop = (monitor2*) observer->get_monitor_by_index(i);
      prop->callback_received_inputs(addent1, addent2);
    }
#endif

    DP->dprint(5, "driver(): input1 = %d, input2 = %d\n", addent1, addent2);

    DP->dprint(5, "driver(): Initialize the local memory to %d and spawn %d processes\n", addent1, addent2);
    _a = addent1;

    for (int i=0; i < addent2; i++) {
      DP->dprint(5, "driver(): Spawning an add1 process (seq %d)\n", i+1);
      sc_spawn( sc_bind(&adder::do_add1, this) );
    }

    DP->dprint(5, "driver(): Notifying the driver event (SC_ZERO_TIME delay)\n");
    driver_event.notify(SC_ZERO_TIME);
  
    DP->dprint(5, "driver(): Suspending until the driver event is notified\n");
    wait(driver_event);

    DP->dprint(5, "driver(): Resume execution after the driver event is notified\n");

    DP->dprint(5, "driver(): notifying the addition event (immediate notification)\n");
    add1_activate_event.notify();

    DP->dprint(5, "driver(): Notifying the driver event (SC_ZERO_TIME delay)\n");
    driver_event.notify(SC_ZERO_TIME);

    DP->dprint(5, "driver(): Suspending until the driver event is notified\n");
    wait(driver_event);

    DP->dprint(5, "driver(): Resume execution after the driver event is notified\n");
  
    DP->dprint(5, "driver(): Writing the result (%d) to the \"result\" port\n", _a);
    result.write(_a);

#ifdef MONITOR_2
    for (int i = 0; i < num_monitors; i++) {
      DP->dprint(5, "driver(): Received inputs. Notifying monitor i=%d\n", i);
      monitor2* prop = (monitor2*) observer->get_monitor_by_index(i);
      prop->callback_sent_answer(_a);
    }
#endif
  }
  
  //Unreachable
  DP->dprint(4, "driver(): Finished execution. The value of the addent is %d\n", _a);
  assert(false);
}

void 
adder::do_add1() {
  DP->dprint(4, "do_add1(): Starting execution. The value of addent is %d\n", _a);
  DP->dprint(6, "do_add1(): Suspending until the waiting event is notified\n");

  wait(add1_activate_event);

  DP->dprint(6, "do_add1(): Resuming execution after waiting event was notified\n");

  (_a) = (_a) + 1;

  DP->dprint(7, "do_add1(): The value of the addent was incremented\n");
 
  DP->dprint(6, "do_add1(): notifying the addition event\n");
  addition_event.notify(); //immediate notification

  DP->dprint(4, "do_add1(): finishing execution. The value of addent is %d\n", _a);
}

