#include "driver.h"


/**
 * The constructor
 */
driver::driver(sc_module_name module_name, int input):sc_module(module_name) {
  initial_number = input;
  running_total = 0;

  // Initialize the output ports to guarantee deterministic simulation
  output1.initialize(0);
  output1.initialize(0);
  
  // This process will not be initialized
  SC_THREAD(generate_task);
  sensitive << clk.pos();
  dont_initialize();

  // Notice that this process will not be initialized
  SC_METHOD(get_result);
  sensitive << result;
  dont_initialize();
}


/**
 * Generates a new problem for the connected arithmetic unit
 */
void 
driver::generate_task() {
  DP->dprint(4, "generate_task(): started execution\n");
  
  for (int i = 0; i < initial_number; i++) {
    
    int to_output1 = running_total;
    int to_output2 = initial_number;

    DP->dprint(6, "generate_task(): writing %d to port \"output1\"\n", to_output1);
    output1.write(to_output1);
    
    DP->dprint(6, "generate_task(): writing %d to port \"output2\"\n", to_output2);
    output2.write(to_output2);

    DP->dprint(5, "generate_task(): suspending until the next clock tick\n");
    wait();

    DP->dprint(5, "generate_task(): resuming after return from \"wait()\"\n");
  } 


  DP->dprint(4, "generate_task(): finished execution\n");

  DP->dprint(2, "generate_task(): %d ^ 2 = %d\n", initial_number, running_total);
  sc_stop();
  
}


/**
 * Checks the result of the operation
 */
void 
driver::get_result() {
  DP->dprint(4, "check_result(): started execution\n");

  DP->dprint(5, "check_result(): reading from port \"result\"\n");

  running_total = result.read();

  DP->dprint(5, "check_result(): the value of the port is %d\n", running_total);

  DP->dprint(4, "check_result(): finished execution\n");
}


