#include "main.h"


int sc_main(int argc, char** argv) {
  
  if (argc == 1) {
    printf("Usage: %s input_number local_verb (0-9) enable_monotors {0, 1} \n", argv[0]);
    printf("       monitor_verb (0-9) num_monotors show_status {0, 1}\n");
    exit(0);
  }

  int to_square = 5;
  if (argc > 1) {
    to_square = atoi(argv[1]);
  }

  int ver_verbosity = 9;
  if (argc > 2) {
    ver_verbosity = atoi(argv[2]);
  }
  DP->set_level(ver_verbosity);

  bool enable_monitors = 0;
  if (argc > 3) {
    enable_monitors = atoi(argv[3]);
  }
  

  int mon_verbosity = 0;
  if (argc > 4) {
    mon_verbosity = atoi(argv[4]);
  }

  int num_monitors = 0;
  if (argc > 5) {
    num_monitors = atoi(argv[5]);
  }

  bool show_status = 0;
  if (argc > 6) {
    show_status = atoi(argv[6]);
  }
  
  for (int i=0; i<argc; i++) {
    DP->dprint(9, "command line argument %i: %s\n", i, argv[i]);
  }



  // The objects
  adder addy("addy", num_monitors);
  driver minnie("driver", to_square);
  sc_clock ticktack("clock", 10, SC_NS, 0.5);
  
  // The "wires"
  sc_signal<int> signal1, signal2, signal3;

  // Connect the model
  addy.input1(signal1);
  addy.input2(signal2);
  addy.result(signal3);

  minnie.output1(signal1);
  minnie.output2(signal2);
  minnie.result(signal3);

  // Connect the clock
  minnie.clk(ticktack);

#ifdef WITH_OBSERVER
  if (enable_monitors) {
    //observer = new ver_observer(mon_verbosity, num_monitors, &addy);
    observer = new local_observer(mon_verbosity, num_monitors, &addy);
  }
#endif

  DP->dprint(2, "Starting simulation\n");

  sc_start();
  
  struct timeval tim;        
  struct rusage ru;        
  getrusage(RUSAGE_SELF, &ru);        
  tim=ru.ru_utime;        
  double t=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;        
  tim=ru.ru_stime;        
  t+=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;

  printf("getrusage time: %.4f seconds\n", t);
  DP->dprint(1, "Number: %d, local verb: %d, monitor: %s\n", to_square, ver_verbosity, 
	     (enable_monitors? "yes":"no"));
  if (enable_monitors) {
    DP->dprint(1, "kernel verb: %d, number of monitors: %d\n", mon_verbosity, num_monitors);
  }

#ifdef WITH_OBSERVER  
  if (enable_monitors) {
    if (show_status) {
      ((local_observer*) observer)->report_status(std::cout);
    }
  }
#endif

  return 0;
}
